<?php
/***************************************************************************
 *                                                                          *
 *				   (c) 2013 Re:Desk Development Team    				   *
 *                                                                          *
 * This  is  commercial  software,  only  users  who have purchased a valid *
 * license  and  accept  to the terms of the  License Agreement can install *
 * and use this program.                                                    *
 *                                                                          *
 ****************************************************************************
 * PLEASE READ THE FULL TEXT  OF THE SOFTWARE  LICENSE   AGREEMENT  IN  THE *
 * "copyright.txt" FILE PROVIDED WITH THIS DISTRIBUTION PACKAGE.            *
 ****************************************************************************/

class MailRetriverCommand extends CConsoleCommand
{
	const OLD_MAIL_DAYS = 1;

	/**
	 * @var $mail_parser MailParser
	 */
	protected $mail_parser;

	function __construct()
	{
	}

	public function run($args)
	{
		Yii::import('application.vendors.*');

		include_once 'MailParser/MailParser.class.php';
		include_once 'MailParser/MailRetriever.class.php';

		$options = array (
			'host'      => Settings::get('email_host'),
			'port'      => Settings::get('email_port'),
			'username'  => Settings::get('email_address'),
			'password'  => Settings::get('email_password'),
			'ssl'       => Settings::get('use_ssl_email'),
		);

		if (Settings::get('timezone'))
			Yii::app()->setTimeZone(Settings::get('timezone'));
		else
			Yii::app()->setTimeZone('Europe/Moscow');

		$this->mail_parser = new MailParser();

		if (empty($options['host']) || empty($options['port']) || empty($options['username']) || empty($options['password']))
			return false;

		$retriever = new MailRetriever($options);

		if ($retriever->status === false)
			return false;

		$this->processMessages($retriever);

		// @FIXME paid accounts
		if (Yii::app()->cache && !Yii::app()->cache->get('expired_license'))
		{
			$accounts = EmailAccounts::model()->findAll();

			if (!empty($accounts))
			{
				foreach ($accounts as $account)
				{
					$options = array (
						'host'       => $account->email_host,
						'port'       => $account->email_port,
						'username'   => $account->email_address,
						'password'   => $account->email_password,
						'ssl'        => $account->use_ssl_email,
					);

					// @FIXME
					$retriever = new MailRetriever($options);
					$this->processMessages($retriever, $account);
				}
			}
		}

        /*$facebook_exists = Settings::get('facebook_page');

        if ($facebook_exists)
            $this->processFacebook();

		$twitter_exists = Settings::get('oauth_token');

		if ($twitter_exists)
			$this->processTwitter();*/

		// cleanup trash
		Ticket::model()->deleteAllByAttributes(array ('folder_id' => TicketFolders::TRASH), 'created_at < (UNIX_TIMESTAMP() - 2592000)');

        // close old tickets
		$close_days = Settings::get('auto_close_days');

		if (!empty($close_days))
			Ticket::model()->updateAll(['is_open' => 0], 'last_staff > last_client AND last_client < UNIX_TIMESTAMP() - :close_days', [':close_days' => 60 * 60 * 24 * $close_days]);

		return true;
	}

	function createTicket($params, EmailAccounts $account = null)
	{
		$body = $params['body'];
		// TODO: print log information

		if (!$params['subject'] && !$body) {
			return null;
		}

		$ticket = new Ticket();
		$ticket->subject = $params['subject'];
		$ticket->author_email = $params['author_email'];
		$ticket->author_name = $params['author_name'];
		$ticket->user_id = 0;
		$ticket->is_open = 1;
		$ticket->account_id = $params['account_id'] ?: 0;
		$ticket->content = $body;

		if (!empty($params['created_at']))
			$ticket->created_at = $params['created_at'];

		if (!empty($params['folder']))
			$ticket->folder_id = $params['folder'];

		if (!empty($params['external_id']) && !empty($params['external_type']))
		{
			$ticket->external_id = $params['external_id'];
			$ticket->external_type = $params['external_type'];
		}

		if (!empty($account) && $account->department)
		{
			$category = TicketCategory::model()->findByPk($account->department);

			if (!empty($category))
				$ticket->category_id = $account->department;
		}
		elseif (!empty($params['category_id']))
			$ticket->category_id = $params['category_id'];

		if (!empty($params['permalink']))
			$ticket->permalink = $params['permalink'];

		/**
		 * @var User $user
		 */
		$user = User::model()->findByAttributes(array('email' => $params['author_email']));

		if (!empty($user))
			$ticket->user_id = $user->id;

		$ticket->save(false);

		// no message
		if (!$body)
			return $ticket;

		// next - create message
		$message = new TicketMessage();
		$message->author_email = $params['author_email'];
		$message->author_name = $params['author_name'];
		$message->message = $body;
		$message->ticket_id = $ticket->id;
		$message->user_id = $ticket->user_id;

		if ($params['external_id'])
		{
			$message->external_id = $params['external_id'];
			$message->external_type = $params['external_type'];
		}

		if ($params['created_at'])
			$message->created_at = $params['created_at'];

		$message->save(false);

		if (!empty($params['attachments']))
		{
			foreach ($params['attachments'] as $attach)
			{
				$attach_add = new MessageAttachment();
				$attach_add->file_name = $attach['file_name'];
				$attach_add->file_type = $attach['mime_type'];
				$attach_add->content = $attach['content'];
				$attach_add->message_id = $message->id;
				$attach_add->size = $attach['size'];
				$attach_add->save(false);
			}
		}

		if (($ticket->folder_id != TicketFolders::TWITTER) && ($ticket->folder_id != TicketFolders::FACEBOOK))
			$ticket->sendNotification($params['author_email'], $params['author_name'], $params['external_from_address'] ? $params['external_from_address'] : '', $params['external_from_name'] ? $params['external_from_name'] : '', $ticket->account_id);

		return $ticket;
	}

	protected function createMessage($params)
	{
		if (!$params['body'] && count($params['attachments']) == 0)
			return null;

		/**
		 * @var Ticket $ticket
		 */
		$ticket = Ticket::model()->findByAttributes(array('tracking_id' => $params['ticket_id']));

		if (!$ticket) {
			//TODO: send notification about absence of the ticket
			return false;
		}

		$assigned_id = 0;

		foreach ($ticket->messages as $message)
		{
			if ($message->assign_id)
			{
				$assigned_id = $message->assign_id;
				break;
			}
		}

		$message = new TicketMessage();
		$message->author_email = $params['author_email'];
		$message->author_name = $params['author_name'];
		$message->message = $params['body'];
		$message->ticket_id = $ticket->id;
		$message->assign_id = $assigned_id;

		if ($params['created_at'])
			$message->created_at = $params['created_at'];

		if ($params['external_id'])
		{
			$message->external_id = $params['external_id'];
			$message->external_type = $params['external_type'];
		}

		/**
		 * @var User $user
		 */
		$user = User::model()->findByAttributes(array('email' => $params['author_email']));

		if (!empty($user))
			$message->user_id = $user->id;

		$message->save();

		if (!empty($params['attachments']))
		{
			foreach ($params['attachments'] as $attach)
			{
				$attach_add = new MessageAttachment();
				$attach_add->file_name = $attach['file_name'];
				$attach_add->file_type = $attach['mime_type'];
				$attach_add->size = mb_strlen($attach['content'], APP_ENCODING);
				$attach_add->content = $attach['content'];
				$attach_add->message_id = $message->id;
				$attach_add->save(false);
			}
		}

		$ticket = $message->ticket;
		$ticket->is_open = 1;
		$ticket->save(false);

		return true;
	}

	public function isSpam(EIMapMessage $mail)
	{
		/** @var SpamFilters[] $filters */
		$filters = SpamFilters::model()->findAll();

		if (!empty($filters))
		{
			foreach ($filters as $filter)
			{
				if ($filter->type == SpamFilters::TYPE_EMAIL)
				{
					$emails = explode(',', $filter->value);
					$emails = array_map('trim', $emails);

					if (in_array($mail->fromAddress, $emails))
						return true;
				}
				else
				{
					if ($filter->type == SpamFilters::TYPE_REGEXP_BODY)
						$string = $mail->textPlain;
					elseif ($filter->type == SpamFilters::TYPE_REGEXP_SUBJECT)
						$string = $mail->subject;
					elseif ($filter->type == SpamFilters::TYPE_REGEXP_EMAIL)
						$string = $mail->fromAddress;

					if (!empty($string))
					{
						if (preg_match('/' . $filter->value . '/is', $string))
							return true;
					}
				}
			}
		}

		return false;
	}

	public function processMessages(MailRetriever $retriever, EmailAccounts $account = null)
	{
		foreach ($retriever->listMessages() as $id)
		{
			$uid = Settings::get('last_uid');

			if (!empty($uid) && $id <= $uid)
				continue;
			else
				Settings::set('last_uid', $id);

			$mail = $retriever->fetchMessage($id);
			$timestamp = strtotime($mail->date);

			$info = $this->mail_parser->extractInfoFromSubject($mail->subject);

			// disable OLD mail messages, for the first start for example
			if ($timestamp && (time() - $timestamp > self::OLD_MAIL_DAYS * 60 * 60 * 24))
			{
				echo "Skip old email...\n";
				continue;
			}

			if (!empty($mail))
			{
				if ($this->isSpam($mail) && Yii::app()->cache && !Yii::app()->cache->get('expired_license'))
				{
					echo "Spam message skipped...\n";
					continue;
				}

				$attachments = $retriever->getAttachments($id);

				$to_model = array ();

				if (!empty($attachments))
				{
					foreach ($attachments as $attach)
					{
						$str = file_get_contents($attach['filepath']);

						$to_model[] = array (
							'content' => $str,
							'mime_type' => $attach['mimetype'],
							'file_name' => $attach['original_name'],
							'size' => strlen($str),
						);

						unset($str);
					}
				}

				$params = array (
					'body' => $this->mail_parser->getReplyBody($mail),
					'subject' => $mail->subject,
					'author_email' => $mail->fromAddress,
					'author_name' => $mail->fromName,
					'attachments' => $to_model,
				);

				if (!empty($account))
				{
					$params['department'] = $account->department;
					$params['external_from_name'] = $account->email_name;
					$params['external_from_address'] = $account->email_address;
					$params['account_id'] = $account->id;
				}

				if (!empty($info['ticket_id']))
				{
					$params['ticket_id'] = $info['ticket_id'];

					$res = $this->createMessage($params);

					if ($res === false)
						$this->createTicket($params, $account);
				}
				else
					$this->createTicket($params, $account);

				// @FIXME - fix delete message
				if (!Settings::get('leave_message'))
					$retriever->deleteMessage($id);
			}
		}
	}

	public function processFacebook()
	{
		// process private messages
        $facebook = FacebookApi::getConnection(Settings::get('facebook_page_token'));

		$max_updated_time = Yii::app()->db->createCommand("SELECT MAX(external_updated_time) FROM " . Ticket::model()->tableName() ." WHERE external_type = '" . TicketMessage::TYPE_FB_POST . "'")->queryScalar();

		if (!$max_updated_time)
			$max_updated_time = 0;

        $request = $facebook->get('/' . Settings::get('facebook_page') . '/conversations?fields=id,snippet,created_time,updated_time,message_count,participants&limit=100');

        $graphObject = $request->getDecodedBody();

        if (!empty($graphObject) && !empty($graphObject['data']))
		{
			foreach ($graphObject['data'] as $row)
			{
				if (strtotime($row['updated_time']) <= $max_updated_time)
					break;

				// disable OLD facebook messages, for the first start for example
				if ((time() - strtotime($row['updated_time']) > self::OLD_MAIL_DAYS * 60 * 60 * 24))
					break;

				$ticket_id = Yii::app()->db->createCommand("SELECT tracking_id FROM " . Ticket::model()->tableName() . " WHERE external_type = '" . TicketMessage::TYPE_FB_POST . "' AND external_id = '" . $row['id'] . "'")->queryScalar();

				if (empty($ticket_id))
				{
					$params = array(
						'body'					=> $row['message'],
						'subject'               => $row['snippet'],
						'author_email'          => $row['participants']['data'][0]['id'],
						'author_name'           => $row['participants']['data'][0]['name'],
						'folder'                => TicketFolders::FACEBOOK,
						'created_at'            => strtotime($row['updated_time']),
						'external_id'           => $row['id'],
						'external_type'         => TicketMessage::TYPE_FB_POST,
						'external_updated_time' => strtotime($row['updated_time']),
						'category_id'			=> 6,
					);

					$ticket = $this->createTicket($params);
					$ticket_id = $ticket->tracking_id;
				}
				else
					$ticket = Ticket::model()->findByAttributes(['tracking_id' => $ticket_id]);

				// Get post comments
				if (!empty($row['message_count']))
				{
					$max_ticket_external_time = Yii::app()->db->createCommand("SELECT MAX(created_at) FROM " . TicketMessage::model()->tableName() . " WHERE ticket_id = '" . $ticket->id . "' AND external_type = '" . TicketMessage::TYPE_FB_MESSAGE . "' ORDER BY id DESC LIMIT 1")->queryScalar();

					$request = $facebook->get("/$row[id]/messages?fields=message,from,created_time&limit=100");

					$messages = $request->getDecodedBody();

					foreach ($messages['data'] as $message)
					{
						if (strtotime($message['created_time']) <= $max_ticket_external_time)
							break;

						if ($message['from']['id'] == Settings::get('facebook_page'))
							continue;

						$params = array(
							'body'          => $message['message'],
							'author_email'  => $message['from']['id'],
							'author_name'   => $message['from']['name'],
							'created_at'    => strtotime($message['created_time']),
							'external_id'   => $message['id'],
							'ticket_id'     => $ticket->tracking_id,
							'external_type' => TicketMessage::TYPE_FB_MESSAGE,
						);

						$this->createMessage($params);
					}

					$ticket->saveAttributes(array('external_updated_time' => strtotime($row['updated_time'])));
				}
			}
		}

		// process tagged posts
		$max_updated_time = Yii::app()->db->createCommand("SELECT MAX(external_updated_time) FROM " . Ticket::model()->tableName() ." WHERE external_type = '" . TicketMessage::TYPE_FB_TAG . "'")->queryScalar() ?: 0;

		if (!$max_updated_time)
			$max_updated_time = 0;

		$request = $facebook->get('/' . Settings::get('facebook_page') . '/tagged?fields=permalink_url,id,message,created_time,updated_time,from,description&limit=100');

		$graphObject = $request->getDecodedBody();

		if (!empty($graphObject) && !empty($graphObject['data']))
		{
			foreach ($graphObject['data'] as $row)
			{
				if (strtotime($row['updated_time']) <= $max_updated_time)
					break;

				// disable OLD facebook messages, for the first start for example
				if ((time() - strtotime($row['updated_time']) > self::OLD_MAIL_DAYS * 60 * 60 * 48))
					continue;

				$ticket_id = Yii::app()->db->createCommand("SELECT tracking_id FROM " . Ticket::model()->tableName() . " WHERE external_type = '" . TicketMessage::TYPE_FB_TAG . "' AND external_id = '" . $row['id'] . "'")->queryScalar();

				if (empty($ticket_id))
				{
					$params = array(
						'subject'               => empty($row['message']) ? $row['description'] : $row['message'],
						'author_email'          => $row['from']['id'],
						'author_name'           => $row['from']['name'],
						'folder'                => TicketFolders::FACEBOOK,
						'created_at'            => strtotime($row['created_time']),
						'external_id'           => $row['id'],
						'external_type'         => TicketMessage::TYPE_FB_TAG,
						'external_updated_time' => strtotime($row['updated_time']),
						'category_id'			=> 6,
						'permalink'				=> $row['permalink_url'],
					);

					$ticket = $this->createTicket($params);
					$ticket_id = $ticket->tracking_id;
				}
				else
					$ticket = Ticket::model()->findByAttributes(['tracking_id' => $ticket_id]);

				$request = $facebook->get("/$row[id]/comments?fields=message,comment_count,from,created_time,updated_time&limit=100&filter=stream&order=reverse_chronological");
				$messages = $request->getDecodedBody();

				// Get post comments
				if (!empty($messages['data']))
				{
					$max_ticket_external_time = Yii::app()->db->createCommand("SELECT MAX(created_at) FROM " . TicketMessage::model()->tableName() . " WHERE ticket_id = '" . $ticket->id . "' AND external_type = '" . TicketMessage::TYPE_FB_COMMENT . "' ORDER BY id DESC LIMIT 1")->queryScalar();

					foreach ($messages['data'] as $message)
					{
						if (strtotime($message['created_time']) <= $max_ticket_external_time)
							continue;

						if ($message['from']['id'] == Settings::get('facebook_page'))
							continue;

						$params = array(
							'body'          => $message['message'],
							'author_email'  => $message['from']['id'],
							'author_name'   => $message['from']['name'],
							'created_at'    => strtotime($message['created_time']),
							'external_id'   => $message['id'],
							'ticket_id'     => $ticket->tracking_id,
							'external_type' => TicketMessage::TYPE_FB_COMMENT,
						);

						$this->createMessage($params);
					}

					$ticket->saveAttributes(array('external_updated_time' => strtotime($row['updated_time'])));
				}
			}
		}

		// process feed posts
		$max_updated_time = Yii::app()->db->createCommand("SELECT MAX(external_updated_time) FROM " . Ticket::model()->tableName() ." WHERE external_type = '" . TicketMessage::TYPE_FB_FEED . "'")->queryScalar() ?: 0;

		if (!$max_updated_time)
			$max_updated_time = 0;

		$request = $facebook->get('/' . Settings::get('facebook_page') . '/feed?fields=permalink_url,id,message,created_time,updated_time,from,description&limit=100');

		$graphObject = $request->getDecodedBody();

		if (!empty($graphObject) && !empty($graphObject['data']))
		{
			foreach ($graphObject['data'] as $row)
			{
				if (strtotime($row['updated_time']) <= $max_updated_time)
					break;

				// disable OLD facebook messages, for the first start for example
				if ((time() - strtotime($row['updated_time']) > self::OLD_MAIL_DAYS * 60 * 60 * 48))
					continue;

				$ticket_id = Yii::app()->db->createCommand("SELECT tracking_id FROM " . Ticket::model()->tableName() . " WHERE external_type = '" . TicketMessage::TYPE_FB_FEED . "' AND external_id = '" . $row['id'] . "'")->queryScalar();

				if (empty($ticket_id))
				{
					$params = array(
						'subject'               => empty($row['message']) ? $row['description'] : $row['message'],
						'author_email'          => $row['from']['id'],
						'author_name'           => $row['from']['name'],
						'folder'                => TicketFolders::FACEBOOK,
						'created_at'            => strtotime($row['created_time']),
						'external_id'           => $row['id'],
						'external_type'         => TicketMessage::TYPE_FB_FEED,
						'external_updated_time' => strtotime($row['updated_time']),
						'category_id'			=> 6,
						'permalink'				=> $row['permalink_url'],
					);

					$ticket = $this->createTicket($params);
					$ticket_id = $ticket->tracking_id;
				}
				else
					$ticket = Ticket::model()->findByAttributes(['tracking_id' => $ticket_id]);

				$request = $facebook->get("/$row[id]/comments?fields=message,comment_count,from,created_time,updated_time&limit=100&filter=stream&order=reverse_chronological");
				$messages = $request->getDecodedBody();

				// Get post comments
				if (!empty($messages['data']))
				{
					$max_ticket_external_time = Yii::app()->db->createCommand("SELECT MAX(created_at) FROM " . TicketMessage::model()->tableName() . " WHERE ticket_id = '" . $ticket->id . "' AND external_type = '" . TicketMessage::TYPE_FB_COMMENT . "' ORDER BY id DESC LIMIT 1")->queryScalar();

					foreach ($messages['data'] as $message)
					{
						if (strtotime($message['created_time']) <= $max_ticket_external_time)
							continue;

						if ($message['from']['id'] == Settings::get('facebook_page'))
							continue;

						$params = array(
							'body'          => $message['message'],
							'author_email'  => $message['from']['id'],
							'author_name'   => $message['from']['name'],
							'created_at'    => strtotime($message['created_time']),
							'external_id'   => $message['id'],
							'ticket_id'     => $ticket->tracking_id,
							'external_type' => TicketMessage::TYPE_FB_COMMENT,
						);

						$this->createMessage($params);
					}

					$ticket->saveAttributes(array('external_updated_time' => strtotime($row['updated_time'])));
				}
			}
		}
    }

	function getFacebookPostAuthor($id, Facebook $facebook)
	{
		$post_author_query = array(
				"user" => "SELECT first_name, last_name FROM user WHERE uid = '$id'",
				"page" => "SELECT name FROM page WHERE page_id = '$id'",
		);

		$results = $facebook->api(array(
				'method'  => 'fql.multiquery',
				'queries' => $post_author_query,
		));

		foreach ($results as $_res)
		{
			if ($_res['name'] == 'page' && !empty($_res['fql_result_set']))
				$author_name = $_res['fql_result_set'][0]['name'];
			else
				$author_name = $_res['fql_result_set'][0]['first_name'] . ' ' . $_res['fql_result_set'][1]['first_name'];

			if ($author_name)
				break;
		}

		return $author_name;
	}

	public function processTwitter()
	{
		$connection = TwitterApi::getConnection();

		$since_id = Yii::app()->db->createCommand("SELECT MAX(tm.external_id) FROM " . TicketMessage::model()->tableName() ." AS tm JOIN " . Ticket::model()->tableName() . " AS t ON t.id = tm.ticket_id WHERE tm.is_worker = 0 AND t.folder_id = " . TicketFolders::TWITTER)->queryScalar();

		$params = array(
			'screen_name' => Settings::get('twitter_account'),
		);

		if (!empty($since_id))
			$params['since_id'] = $since_id;

		$response = $connection->get('statuses/mentions_timeline', $params);
		$response_messages = $connection->get('direct_messages', $params);

		if (!$response)
			$response = array ();

		if (!$response_messages)
			$response_messages = array ();

		$response = array_merge($response, $response_messages);

		if (!empty($response))
		{
			foreach ($response as $k => $item)
			{
				$tweet_date = strtotime($item->created_at);
				$ticket_id = false;

				// skip old tweets
				if ($tweet_date && (time() - $tweet_date > self::OLD_MAIL_DAYS * 60 * 60 * 24))
					continue;

				// this is DM
				if ($item->sender)
				{
					$author_name = $item->sender->name;
					$author_email = $item->sender->screen_name;

					// check that it DM conversation
					$ticket_id = Yii::app()->db->createCommand("SELECT t.tracking_id FROM " . TicketMessage::model()->tableName() . " as tm JOIN " . Ticket::model()->tableName() . " as t ON t.id = tm.ticket_id WHERE tm.external_type = '" . TicketMessage::TYPE_DM . "' AND tm.author_email = '" . $item->sender->screen_name . "'")->queryScalar();
				}
				// else - @mention
				else
				{
					$author_name = $item->user->name;
					$author_email = $item->user->screen_name;

					if (!empty($item->in_reply_to_status_id))
						$ticket_id = Yii::app()->db->createCommand("SELECT t.tracking_id FROM " . TicketMessage::model()->tableName() . " as tm JOIN " . Ticket::model()->tableName() . " as t ON t.id = tm.ticket_id WHERE tm.external_type IN ('" . TicketMessage::TYPE_TWEET . "', '" . TicketMessage::TYPE_DM . "') AND tm.external_id = " . $item->in_reply_to_status_id)->queryScalar();
				}

				// new ticket, no conversation
				if (empty($ticket_id))
				{
					$params = array(
						'body' => $item->text,
						'subject' => $item->text,
						'author_email' => $author_email,
						'author_name' => $author_name,
						'folder' => TicketFolders::TWITTER,
						'created_at' => $tweet_date,
						'external_id' => $item->id,
						'external_type' => empty($item->recipient) ? TicketMessage::TYPE_TWEET : TicketMessage::TYPE_DM,
					);

					$this->createTicket($params);
				}
				else
				{
					$params = array(
						'body' => $item->text,
						'author_email' => $author_email,
						'author_name' => $author_name,
						'created_at' => $tweet_date,
						'external_id' => $item->id,
						'ticket_id' => $ticket_id,
						'external_type' => empty($item->recipient) ? TicketMessage::TYPE_TWEET : TicketMessage::TYPE_DM,
					);

					$this->createMessage($params);
				}
			}
		}
	}
}

if (!function_exists('sendAppMail'))
{
	function sendAppMail(YiiMailMessage $mail, EmailAccounts $account = null)
	{
		// Send messages block, author FROM settings
		Yii::app()->mail->transportType = 'smtp';

		if (empty($account)) {
			$host = Settings::get('smtp_host');

			if (!empty($host)) {
				Yii::app()->mail->transportOptions = array(
					'host' => Settings::get('smtp_host'),
					'username' => Settings::get('email_address'),
					'password' => Settings::get('email_password'),
					'port' => Settings::get('smtp_port'),
					'encryption' => Settings::get('smtp_encryption'),
				);
			} else {
				$mail->setFrom('support@re-desk.com');

				Yii::app()->mail->transportOptions = array(
					'host' => 'smtp.yandex.ru',
					'username' => 'support@re-desk.com',
					'password' => 'dsjdk&^21nmnnma',
					'port' => 465,
					'encryption' => 'ssl',
				);
			}
		} else {
			// Send messages block, author FROM settings
			Yii::app()->mail->transportType = 'smtp';

			Yii::app()->mail->transportOptions = array(
				'host' => $account->smtp_host,
				'username' => $account->email_address,
				'password' => $account->email_password,
				'port' => $account->smtp_port,
				'encryption' => $account->smtp_encryption
			);

			$mail->account_id = $account->id;
		}

		return Yii::app()->mail->send($mail);
	}
}