<?php

class UpgradeManager extends CDbMigration
{
	public function getReleases()
	{
		return array (
			'1.1.0' => '_install_1_1_0',
			'2.0.1' => '_install_2_0_1',
		);
	}

	public function installPatch($patch)
	{
		return call_user_func(array($this, $patch));
	}

	/**
	 * UPGRADE FUNCTIONS
	 */

	/**
	 * Release 1.0.1 => 1.1.0
	 */
	private function _install_1_1_0()
	{
		$api_key = new Settings();
		$api_key->label = 'api_key';
		$api_key->name = 'Api key (e-commerce integration)';
		$api_key->group_id = 1;
		$api_key->value = Ticket::model()->getTracking(12);
		$api_key->visible = 1;
		$api_key->save();

		$api_key = new Settings();
		$api_key->label = 'leave_message';
		$api_key->name = 'Leave a copy of the messages on the email server';
		$api_key->group_id = 2;
		$api_key->value = 0;
		$api_key->visible = 1;
		$api_key->save();

		$api_key = new Settings();
		$api_key->label = 'api_url';
		$api_key->name = 'URL of integrated website (shopping cart for example)';
		$api_key->group_id = 1;
		$api_key->value = 0;
		$api_key->visible = 1;
		$api_key->save();

		$api_key = new Settings();
		$api_key->label = 'email_fetching';
		$api_key->name = '';
		$api_key->group_id = 1;
		$api_key->value = 0;
		$api_key->visible = 0;
		$api_key->save();

		$api_key = new Settings();
		$api_key->label = 'version';
		$api_key->name = '';
		$api_key->group_id = 1;
		$api_key->value = "1.1.0";
		$api_key->visible = 0;
		$api_key->save();

		$this->createTable(
			'tbl_spam_filters',
			array (
				'id'    => 'pk',
				'type'  => 'VARCHAR(32) NOT NULL',
				'value' => 'VARCHAR(255) NOT NULL',
			)
		);

		$this->update('AuthItem', array ('description' => 'Department Manager'), 'name = "Ticket.Index"');
	}

	private function _install_2_0_1()
	{
		// new access property
		$auth_item = new AuthItem();
		$auth_item->name = 'Ticket.ViewCustomerData';
		$auth_item->type = 0;
		$auth_item->description = 'View customer data';
		$auth_item->data = 'N;';
		$auth_item->save();

		// add indexes
		$this->createIndex('ticket_author_email', 'tbl_ticket', 'author_email');
		$this->createIndex('message_author_email', 'tbl_ticket_message', 'author_email');

		$this->createTable(
			'tbl_email_templates',
			array (
				'template_code' => 'VARCHAR(32) NOT NULL',
				'name'          => 'VARCHAR(255) NOT NULL',
				'subject'       => 'VARCHAR(512) NOT NULL',
				'template'      => 'MEDIUMTEXT NOT NULL',
				'lang_code'     => 'VARCHAR(4) NOT NULL DEFAULT \'en\'',
				'PRIMARY KEY (`template_code`,`lang_code`)'
			)
		);

		$this->insert('tbl_settings', array ('group_id' => 1, 'name' => 'avail languages', 'label' => 'languages', 'visible' => 1, 'avail_values' => '["en"]'));
		$this->insert('tbl_settings', array ('group_id' => 1, 'name' => 'close ticket', 'label' => 'close_ticket', 'visible' => 1, 'value' => 1));

		$this->execute("insert into `tbl_email_templates` (`template_code`, `name`, `subject`, `template`, `lang_code`) values('new_message', 'new message', 'RE: {TICKET_SUBJECT}', '{EMAIL_CONTENT}\r\n\r\n--\r\nYou can see this communication ticket and message online <a href=\"{TICKET_URL}\">there</a>.', 'en');");
		$this->execute("insert into `tbl_email_templates` (`template_code`, `name`, `subject`, `template`, `lang_code`) values('recovery', 'Recovery', '{COMPANY_NAME}: Password Recovery ', 'You have requested a password recovery. \r\n\r\nPlease follow this link to confirm your password recovery procedure:\r\n{ACTIVATION_URL}.\r\n\r\n----------\r\nPowered by <a href=\"http://re-desk.com\">Re:Desk software</a>', 'en');");
		$this->execute("insert into `tbl_email_templates` (`template_code`, `name`, `subject`, `template`, `lang_code`) values('registration', 'Registration', 'Registration: welcome to {COMPANY_NAME}''s help desk', 'To activate your account please click to <a href=''{ACTIVATION_URL}''>this link</a>\r\n\r\nThank you for choosing us!\r\n----------\r\nPowered by <a href=\"http://re-desk.com\">Re:Desk software</a>', 'en');");
		$this->execute("insert into `tbl_email_templates` (`template_code`, `name`, `subject`, `template`, `lang_code`) values ('ticket_created', 'staff ticket created', 'New ticket \"{TICKET_SUBJECT}\" has been created', 'Hi.\r\n\r\nNew ticket {TICKET_SUBJECT} created.\r\n\r\n{EMAIL_CONTENT}\r\n\r\nYou can see this communication ticket online <a href=\"{TICKET_URL}\">there</a>.\r\n\r\n{COMPANY_NAME} team.\r\n\r\n----------\r\nPowered by <a href=\"http://re-desk.com\">Re:Desk software</a>', 'en');");
		$this->execute("insert into `tbl_email_templates` (`template_code`, `name`, `subject`, `template`, `lang_code`) values ('new_ticket_notification', 'new ticket', 'Helpdesk System - Your ticket {TICKET_ID} has been created', 'Hi {TICKET_AUTHOR},\r\n\r\nThank you for contacting {COMPANY_NAME}''s support team!\r\nWe''ve received your ticket, and it''s been entered into our system with the ID #{TICKET_ID}.\r\n\r\nYou can see this communication ticket online <a href=\"{TICKET_URL}\">there</a>.\r\n\r\n{COMPANY_NAME} team.\r\n\r\n----------\r\nPowered by <a href=\"http://re-desk.com\">Re:Desk software</a>', 'en');");

		$this->addColumn('tbl_users', 'send_notifications', "SMALLINT(1) NOT NULL DEFAULT '0'");
		$this->addColumn('tbl_ticket_category', 'translations', 'TEXT NOT NULL DEFAULT ""');

		/** @var TicketCategory[] $categories */
		$categories = TicketCategory::model()->findAll();
		$languages = Settings::get('languages');

		foreach ($categories as $category)
		{
			$translations = array();

			foreach ($languages as $lang_code)
			{
				$translations[$lang_code] = $category->name;
			}

			$category->translations = CJSON::encode($translations);
			$category->save();
		}

		$this->createTable(
			'tbl_email_accounts',
			array (
				'id' => 'INT(11) NOT NULL AUTO_INCREMENT',
				'email_address' => 'VARCHAR(512) NOT NULL',
				'email_password' => 'VARCHAR(512) NOT NULL',
				'email_transport' => 'VARCHAR(256) NOT NULL',
				'email_host' => 'VARCHAR(256) NOT NULL',
				'email_port' => 'VARCHAR(256) NOT NULL',
				'email_name' => 'VARCHAR(256) NOT NULL',
				'use_ssl_email' => "TINYINT(1) NOT NULL DEFAULT '0'",
				'leave_copy' => "TINYINT(1) NOT NULL DEFAULT '0'",
				'department' => "INT(11) NOT NULL DEFAULT '0'",
				'PRIMARY KEY (`id`)'
			)
		);

		// fix user categories
		$wrong_cat_ids = Yii::app()->db->createCommand('
			SELECT tc.id FROM tbl_user_categories as tc
			LEFT JOIN tbl_users as u ON u.id = tc.user_id
			WHERE u.id IS NULL
		')->queryColumn();

		if (!empty($cat_ids))
			$this->delete('tbl_user_categories', array ('in', 'id', $wrong_cat_ids));

		$this->addForeignKey('user_categories', 'tbl_user_categories', 'user_id', 'tbl_users', 'id', 'CASCADE', 'CASCADE');

		$this->insert('tbl_settings', array ('group_id' => 1, 'name' => 'auto response', 'label' => 'auto_response', 'visible' => 1, 'value' => 1));
		$this->insert('tbl_settings', array ('group_id' => 1, 'name' => 'timezone', 'label' => 'timezone', 'visible' => 1, 'value' => ''));
		$this->insert('tbl_settings', array ('group_id' => 1, 'name' => 'worktime', 'label' => 'worktime', 'visible' => 1, 'value' => ''));

		$this->update('tbl_settings', array ('value' => '2.0.1'), 'label = "version"');

		Yii::app()->cache->flush();
	}

	private function _install_2_1_1()
	{
		$this->addColumn('tbl_ticket_message', 'external_id', "VARCHAR(64) NOT NULL DEFAULT '0'");
		$this->addColumn('tbl_ticket_message', 'external_type', "VARCHAR(16) NOT NULL DEFAULT ''");

		$this->createIndex('external', 'tbl_ticket_message', '`external_id`, `external_type`');

		$this->addColumn('tbl_ticket', 'external_id', "VARCHAR(64) NOT NULL DEFAULT '0'");
		$this->addColumn('tbl_ticket', 'external_type', "VARCHAR(16) NOT NULL DEFAULT ''");
		$this->addColumn('tbl_ticket', 'external_updated_time', "INT(11) NOT NULL DEFAULT '0'");

		$this->createIndex('external', 'tbl_ticket', '`external_id`, `external_type`');
		$this->createIndex('external_updated_time', 'tbl_ticket', 'external_updated_time');

		$this->insert('tbl_ticket_folders', array ('id' => 4, 'name' => 'Twitter', 'code' => 'twitter'));
		$this->insert('tbl_ticket_folders', array ('id' => 5, 'name' => 'Facebook', 'code' => 'facebook'));

		$this->insert('tbl_settings_group', array ('id' => 3, 'name' => 'twitter settings'));

		$this->insert('tbl_settings', array ('group_id' => 3, 'name' => 'twitter account', 'label' => 'twitter_account', 'visible' => 1));
		$this->insert('tbl_settings', array ('group_id' => 3, 'name' => 'consumer key', 'label' => 'consumer_key', 'visible' => 1));
		$this->insert('tbl_settings', array ('group_id' => 3, 'name' => 'consumer secret', 'label' => 'consumer_secret', 'visible' => 1));
		$this->insert('tbl_settings', array ('group_id' => 3, 'name' => 'oauth token', 'label' => 'oauth_token', 'visible' => 1));
		$this->insert('tbl_settings', array ('group_id' => 3, 'name' => 'oauth token secret', 'label' => 'oauth_token_secret', 'visible' => 1));

		$this->insert('tbl_settings_group', array ('id' => 4, 'name' => 'facebook settings'));

		$this->insert('tbl_settings', array ('group_id' => 4, 'name' => 'facebook page', 'label' => 'facebook_page', 'visible' => 1));
		$this->insert('tbl_settings', array ('group_id' => 4, 'name' => 'facebook token', 'label' => 'facebook_token', 'visible' => 0));

	}
}