<?php
/***************************************************************************
 *                                                                          *
 *				   (c) 2013 Re:Desk Development Team    				   *
 *                                                                          *
 * This  is  commercial  software,  only  users  who have purchased a valid *
 * license  and  accept  to the terms of the  License Agreement can install *
 * and use this program.                                                    *
 *                                                                          *
 ****************************************************************************
 * PLEASE READ THE FULL TEXT  OF THE SOFTWARE  LICENSE   AGREEMENT  IN  THE *
 * "copyright.txt" FILE PROVIDED WITH THIS DISTRIBUTION PACKAGE.            *
 ****************************************************************************/

/**
 * This is the model class for table "tbl_ticket_message_attachment".
 *
 * The followings are the available columns in table 'tbl_ticket_message_attachment':
 * @property integer $id
 * @property integer $message_id
 * @property string $file_name
 * @property string $file_type
 * @property string $content
 * @property string $size
 *
 * The followings are the available model relations:
 * @property TicketMessage $message
 */
class MessageAttachment extends CActiveRecord
{
	/**
	 * Returns the static model of the specified AR class.
	 * @param string $className active record class name.
	 * @return MessageAttachment the static model class
	 */
	public static function model($className=__CLASS__)
	{
		return parent::model($className);
	}

	/**
	 * @return string the associated database table name
	 */
	public function tableName()
	{
		return 'tbl_ticket_message_attachment';
	}

	/**
	 * @return array validation rules for model attributes.
	 */
	public function rules()
	{
		// NOTE: you should only define rules for those attributes that
		// will receive user inputs.
		return array(
			array('message_id, file_name, file_type', 'required'),
			array('message_id', 'numerical', 'integerOnly'=>true),
			array('file_name', 'length', 'max'=>512),
			array('file_type', 'length', 'max'=>64),
			// The following rule is used by search().
			// Please remove those attributes that should not be searched.
			array('id, message_id, file_name, file_type', 'safe', 'on'=>'search'),
		);
	}

	/**
	 * @return array relational rules.
	 */
	public function relations()
	{
		// NOTE: you may need to adjust the relation name and the related
		// class name for the relations automatically generated below.
		return array(
			'message' => array(self::BELONGS_TO, 'TicketMessage', 'message_id'),
		);
	}

	/**
	 * @return array customized attribute labels (name=>label)
	 */
	public function attributeLabels()
	{
		return array(
			'id' => 'ID',
			'message_id' => 'Message',
			'file_name' => 'File Name',
			'file_type' => 'File Type',
		);
	}

	/**
	 * Retrieves a list of models based on the current search/filter conditions.
	 * @return CActiveDataProvider the data provider that can return the models based on the search/filter conditions.
	 */
	public function search()
	{
		// Warning: Please modify the following code to remove attributes that
		// should not be searched.

		$criteria=new CDbCriteria;

		$criteria->compare('id',$this->id);
		$criteria->compare('message_id',$this->message_id);
		$criteria->compare('file_name',$this->file_name,true);
		$criteria->compare('file_type',$this->file_type,true);

		return new CActiveDataProvider($this, array(
			'criteria'=>$criteria,
		));
	}

	public function getSize()
	{
		if ($this->size <= 1024)
			return '1 kb';
		elseif ($this->size <= 1024 * 1024)
			return ceil($this->size / 1024) . ' kb';
		else
			return round(($this->size / (1024 * 1024)), 2) . ' mb';
	}

	public function isImage()
	{
		$image_types = array (
			'image/png',
			'image/jpg',
			'image/jpeg',
			'image/gif',
		);

		return in_array($this->file_type, $image_types);
	}

	public function showImage()
	{
		list(, $file_type) = explode('/', $this->file_type);
		$img_path = Yii::getPathOfAlias('application.runtime') . '/' . $this->id . '.' . $file_type;
		file_put_contents($img_path, $this->content);

		/**
		 * @var Image $image
		 */
		$image = Yii::app()->image->load($img_path);
		if (!empty($_REQUEST['width']))
			$image->resize($_REQUEST['width'], 200);

		$image->render();
		return unlink($img_path);
	}
}